from __future__ import print_function

from tt.dataaccess.utils import get_data_store
from tt.dateutils.dateutils import *
from tt.actions.utils import reportingutils


def action_csv():
    sep = '|'
    data = get_data_store().load()
    work = data['work']

    for item in work:
        if 'end' in item:
            notes = reportingutils.get_notes_from_workitem(item)
            duration = parse_isotime(item['end']) - parse_isotime(item['start'])
            duration_total = reportingutils.remove_seconds(duration)
            date = reportingutils.extract_day(item['start'])
            name = item['name']
            start = format_csv_time(item['start'])
            end = format_csv_time(item['end'])
            tags = ''
            if 'tags' in item:
                tags = item['tags']
            print_elements(date, name, start, end, duration_total, notes, tags, sep)


def print_elements(date, name, start, end, total_duration, notes, tags, sep):
    # print(date, sep, name, sep, start, sep, end, sep, total_duration, sep, tags, sep, notes)
    print(date, sep, start, sep, end, sep, '', sep, notes, sep, name)


def format_csv_time(somedatetime):
    local_dt = isotime_utc_to_local(somedatetime)
    return local_dt.strftime('%H:%M')



from tt.dateutils.dateutils import *


def get_notes_from_workitem(item):
    notes = ''
    if 'notes' in item:
        for note in item['notes']:
            notes += note + ' ; '
    return notes


def extract_day_custom_formatter(datetime_local_tz, format_string):
    local_dt = isotime_utc_to_local(datetime_local_tz)
    return local_dt.strftime(format_string)


def extract_day(datetime_local_tz):
    return extract_day_custom_formatter(datetime_local_tz, '%Y-%m-%d')


def remove_seconds(timedelta):
    return ':'.join(str(timedelta).split(':')[:2])





from unittest import TestCase
from tt.dataaccess.utils import *


class TestDataaccess(TestCase):

    def test_basic_json_datastore_creation(self):
        datastore = get_data_store()
        self.assertIsNotNone(datastore, 'Should not be none, but is')

    def test_loading_json_datastore(self):
        datastore = get_data_store()
        data = datastore.load()
        self.assertIsNotNone(data, 'Should not be none, but is')
        self.assertIsNotNone(data['work'], 'Data should have empty work list, but doesn\'t')

    def test_wrong_datastore_type_generates_exception(self):
        with self.assertRaises(NonexistentDatasource):
            datastore = get_data_store("XML")



from tt.dataaccess.utils import get_data_store
from tt.actions.utils.utils import ensure_working


def action_tag(tags):
    data = get_data_store().load()

    ensure_working(data)

    current = data['work'][-1]

    current['tags'] = set(current.get('tags') or [])
    current['tags'].update(tags)
    current['tags'] = list(current['tags'])

    get_data_store().dump(data)

    tag_count = len(tags)
    print("Okay, tagged current work with %d tag%s."
          % (tag_count, "s" if tag_count > 1 else ""))



# coding: utf-8
from __future__ import print_function

import sys

from tt.colors.colors import Colorizer

from tt.dateutils.dateutils import to_datetime

from tt.exceptz.exceptz import BadArguments
from tt.exceptz.exceptz import TIError

from tt.actions.utils import help

from tt.actions.write import edit
from tt.actions.write import start
from tt.actions.write import stop
from tt.actions.write import tag
from tt.actions.write import note

from tt.actions.read import log
from tt.actions.read import csv
from tt.actions.read import report
from tt.actions.read import calview
from tt.actions.read import status


def parse_args(argv=sys.argv):

    colorizer = Colorizer(True)
    if '--no-color' in argv:
        colorizer.set_use_color(False)
        argv.remove('--no-color')

    # prog = argv[0]
    if len(argv) == 1:
        raise BadArguments("You must specify a command.")

    head = argv[1]
    tail = argv[2:]

    if head in ['-h', '--help', 'h', 'help']:
        fn = help.print_help
        args = {}

    elif head in ['edit']:
        fn = edit.action_edit
        args = {}

    elif head in ['start']:
        if not tail or len(tail) != 2:
            raise BadArguments(
                'Please provide a name for the activity and the start time, like so:\n$ tt start project 14:15')

        fn = start.action_start
        args = {
            'colorizer': colorizer,
            'name': tail[0],
            'time': to_datetime(' '.join(tail[1:])),
        }

    elif head in ['stop']:
        fn = stop.action_stop
        args = {'colorizer': colorizer, 'time': to_datetime(' '.join(tail))}

    elif head in ['status']:
        fn = status.action_status
        args = {'colorizer': colorizer}

    elif head in ['log']:
        fn = log.action_log
        args = {'period': tail[0] if tail else None}

    elif head in ['csv']:
        fn = csv.action_csv
        args = {}

    elif head in ['report']:
        fn = report.action_report
        if not tail:
            raise BadArguments('Please provide the name of the activity for which to generate the report')
        args = {'colorizer': colorizer, 'activity': tail[0]}
        
    elif head in ['calview']:
        fn = calview.action_calview
        if not tail:
            raise BadArguments(
                'Please provide the month [optionally followed by the year] for which to generate the activity report')
        args = {'colorizer': colorizer, 'month': tail[0], 'year': tail[1] if len(tail) > 1 else None}

    elif head in ['tag']:
        if not tail:
            raise BadArguments("Please provide at least one tag to add.")

        fn = tag.action_tag
        args = {'tags': tail}

    elif head in ['note']:
        if not tail:
            raise BadArguments("Please provide some text to be noted.")

        fn = note.action_note
        args = {'colorizer': colorizer, 'content': ' '.join(tail)}

    else:
        raise BadArguments("I don't understand %r" % (head,))

    return fn, args


def main():
    try:
        fn, args = parse_args()
        fn(**args)
    except TIError as e:
        msg = str(e) if len(str(e)) > 0 else __doc__
        print(msg, file=sys.stderr)
        sys.exit(1)


if __name__ == '__main__':
    main()



import os

from tt.datasources.jsondatasource import JsonStore
from tt.exceptz.exceptz import NonexistentDatasource

default_datasource_type = 'JSON'

json_default_env_var_name = 'SHEET_FILE'
json_default_db_location = '~/.tt-sheet.json'


def get_data_store(type=default_datasource_type):
    if type == "JSON":
        return JsonStore(os.getenv(json_default_env_var_name, None) or
                  os.path.expanduser(json_default_db_location))
    else:
        raise NonexistentDatasource('App only supports JSON datasources at the moment')



from __future__ import print_function
from __future__ import absolute_import
from __future__ import unicode_literals

import calendar

from collections import defaultdict

from tt.dateutils.dateutils import *
from tt.dataaccess.utils import get_data_store
from tt.actions.utils import reportingutils
def action_calview(colorizer, month, year):
    report = generate_day_based_report()
    year =  get_current_year_local_tz() if year is None else year

    print('Displaying all entries for ', colorizer.yellow(year+'-'+month),  ' grouped by day:', sep='')

    month_cal = calendar.monthcalendar(int(year),int(month))

    # "normal" beeings don't work on weekends (isSevenDayWeek = false default),
    # in case you like working on saturdays or sundays calview will determine if something was logged
    # if that's the case print the Weekend also in calview, else stay sane with Monday till Friday
    isSevenDayWeek = bool(False)
    for week in range(len(month_cal)):
        weekdays = month_cal[week]
        activity_saturday = get_activity(report, 0, weekdays[5], year, month)
        activity_sunday = get_activity(report, 0, weekdays[6], year, month)
        if activity_saturday != "" or activity_sunday != "":
            isSevenDayWeek = bool(True)

    header = "|         " + colorizer.yellow("Monday") + "           |         " + colorizer.yellow(
        "Tuesday") + "          |         " + colorizer.yellow("Wednesday") + "        |         " + colorizer.yellow(
        "Thursday") + "         |           " + colorizer.yellow("Friday") + "         |"
    delimiter = "+--------------------------+--------------------------+--------------------------+--------------------------+--------------------------+"
    weekEnd = 5

    if (isSevenDayWeek):
        header += "          " + colorizer.yellow("Saturday") + "        |           " + colorizer.yellow(
            "Sunday") + "         |"
        delimiter += "--------------------------+--------------------------+"
        weekEnd = 7

    print(delimiter)
    print(header)
    print(delimiter)
    for week in range(len(month_cal)):
        weekdays = month_cal[week]
        #skip empty weeks completely
        if weekdays[0] == 0 and weekdays[weekEnd - 1] == 0:
            continue
        print("",  end="|")
        for day_index in range(len(weekdays[0:weekEnd])):
            day_cell_header=""
            if weekdays[day_index] != 0:
                day_cell_header=" "+ str(year)+"-"+month.zfill(2)+"-"+str(weekdays[day_index]).zfill(2)
            print(colorizer.blue(day_cell_header.ljust(26, ' ')),  end="|")
        print()
        print(delimiter)
        print_week_activity(colorizer, weekdays[0:weekEnd],  5,  report,  year,  month)
        #print("WEEK DONE")
        print(delimiter)
        

def print_week_activity(colorizer, current_week,  height_in_rows,  report,  year,  month):
   for curr_row in range(height_in_rows):
       print_activity_at_index(colorizer, curr_row,  current_week, report,  year,  month)
       print("") #one week row done

def print_activity_at_index(colorizer, curr_row,  current_week,  report,  year,  month):
    print("",  end="|")    
    for day_index in range(len(current_week)):
        if current_week[day_index] == 0:
            print("".rjust(26, ' '),  end="|")
        else:
            activity_str =  get_activity(report,  curr_row, current_week[day_index],  year,  month)
            print(colorizer.green(activity_str.rjust(26, ' ')),  end="|")

def get_activity(report,  curr_row,  day_key,  year,  month):
    report_key=str(year)+"-"+month.zfill(2)+"-"+str(day_key).zfill(2)
    activity = ""
    try:
        activity_dict = report[report_key]
        if curr_row < len(list(activity_dict)):
            activity_key = list(activity_dict)[curr_row]
            activity_duration = report[report_key][activity_key]
            activity = activity_key + " : " + format_time(activity_duration) + " "
    finally:
        return activity

def generate_day_based_report():
    data = get_data_store().load()
    work = data['work']
    report = dict()
    for item in work:
        day = reportingutils.extract_day(item['start'])
        end_time = parse_isotime(item['end']) if 'end' in item else datetime.utcnow()
        duration = end_time - parse_isotime(item['start'])
        try:
            report[day]
        except KeyError:
            report[day] = defaultdict(lambda: timedelta())

        item_name = item['name'] if 'end' in item else "~wip~" + item['name']
        report[day][item_name] += duration
        #print ('report[', day,  "][", item['name'], "]=" ,  report[day][item['name']])
    return report

def format_time(duration_timedelta):
    return format_time_seconds(duration_timedelta.seconds)


def format_time_seconds(duration_secs ):
    hours, rem = divmod(duration_secs, 3600)
    mins, secs = divmod(rem, 60)
    formatted_time_str = str(hours).rjust(2, str('0')) + ':' + str(mins).rjust(2, str('0'))
    return formatted_time_str
    



from unittest import TestCase
from lark import Lark


class TestLarkParser(TestCase):

    def test_parse_tree_creation(self):
        timeline_grammar = """
            start: instruction+
            instruction: timeentry NEWLINE billingstatus
            
            timeentry: "- " time
            billingstatus: billing
                        
            billing: ( DONOTBILL | BILL )
            time: DIGIT ~ 2 ":" DIGIT ~ 2
            
            DONOTBILL: "do"
            BILL: "bill"
            DIGIT: "0".."9" 
            %import common.NEWLINE
            %import common.WS
            %import common.WS_INLINE
            %ignore WS
        """
        timeline_example = """
            -  16:00
                        bill
            -  18:00
                   do
        """
        parser = Lark(timeline_grammar)
        lark_parse_tree = parser.parse(timeline_example)

        for entry in lark_parse_tree.children:
            print(entry)

        self.assertTrue(len(lark_parse_tree.children) == 2)

        sample_grammar = """
                start: instruction+
                instruction: MOVEMENT NUMBER            -> movement
                           | "c" COLOR [COLOR]          -> change_color
                           | "fill" code_block          -> fill
                           | "repeat" NUMBER code_block -> repeat
                code_block: "{" instruction+ "}"
                MOVEMENT: "f"|"b"|"l"|"r"
                COLOR: LETTER+
                %import common.LETTER
                %import common.INT -> NUMBER
                %import common.WS
            """



def print_help():
    print('tt is a simple command-line time tracking tool.')
    print()
    print('It stores all the information you add in a JSON-formatted time entry database, located  \n'
          'either at the location you specify in the environment variable SHEET_FILE, or by default \n'
          'in ${HOME}/.tt-sheet.json, in case you don\'t set your SHEET_FILE.')
    print()
    print('Usage:')
    print('tt COMMAND [PARAM_0] .. [PARAM_n]')
    print()
    print('Commands: ')
    print('  start [WORK_PACKAGE] [STARTING_TIME] \n'
          '    Description:\n'
          '      Opens a new work package with the supplied name, starting at the supplied time\n'
          '    Examples:\n'
          '      tt start cleaning 10:30\n'
          '      tt start cleaning 1030\n'
          '      tt start cleaning now')
    print()
    print('  stop [END_TIME]\n'
          '    Description:\n'
          '      Closes an open work package at the supplied time\n'
          '    Examples:\n'
          '      tt stop 10:30\n'
          '      tt stop 1030\n'
          '      tt stop now')
    print()
    print('  note [TEXT]\n'
          '    Description:\n'
          '      Adds a note to an open work package\n'
          '    Examples:\n'
          '      tt note x\n'
          '      tt note \'The quick brown fox humps the lazy bear\'')

    print()
    print('  tag [TAG0] [TAG1] [TAG2]\n'
          '    Description:\n'
          '      Adds a list of tags to an open work package\n'
          '    Examples:\n'
          '      tt tag TAGNAME\n'
          '      tt tag \'housework\' \'#makeherproud\'')

    print()
    print('  edit \n'
          '    Description:\n'
          '      Opens your time entry database located at ${SHEET_FILE} on an editor of your choosing. The editor\n'
          '      needs to be specified by setting another environment variable: export EDITOR=\'vim\'\n'
          '      Saving and exiting the editor, updates the time entry db with the new info.\n'
          '      Graphical text editors, such as kate, gedit or sublime ar supported just as well.\n'
          '    Examples:\n'
          '      tt edit')

    print()
    print('  csv \n'
          '    Description:\n'
          '      Lists all your individual entries in a comma-separated format, for ease of import into spreadsheet\n'
          '      editors such as LibreOffice Sheets. The separator is the pipe symbol.\n'
          '    Examples:\n'
          '      tt csv\n'
          '      tt csv --nocolor > /tmp/allentries.csv ; libreoffice /tmp/allentries.csv')

    print()
    print('  status \n'
          '    Description:\n'
          '      Shows all information pertaining to an open work package or an appropriate message, if one cannot\n'
          '      be found. \n'
          '    Examples:\n'
          '      tt status\n'
          '      tt status --nocolor')
    print()
    print('  report [WORK_PACKAGE] \n'
          '    Description:\n'
          '      Creates an aggregated daily report for the work package you specify as parameter. It creates one\n'
          '      aggregated entry per day, based on the entire database content. If you need to restrict the report\n'
          '      to certain periods, such as a specific month, feel free to pipe the output through grep or other\n'
          '      cli tools. \n'
          '    Examples:\n'
          '      tt report cleaning\n'
          '      tt report cleaning --nocolor\n'
          '      tt report cleaning --nocolor | grep 2019-03 ')
    print()
    print('  calview [MONTH] [YEAR] \n'
          '    Description:\n'
          '      Renders a monthly workday calendar (Monday-Friday) with daily aggregated package durations.\n'
          '      The [YEAR] parameter is optional; if omitted, it defaults to the current year. \n'
          '    Examples:\n'
          '      tt calview 12\n'
          '      tt calview 11 --nocolor\n'
          '      tt calview 10 2030')
    print()
    print('For the full documentation, check out http://github.com/dribnif/tt')



from tt.dataaccess.utils import get_data_store
from tt.actions.utils.utils import ensure_working
from tt.actions.utils.utils import ensure_end_after_start
from tt.dateutils.dateutils import formatted_str_for_isotime_str


def action_stop(colorizer, time):
    data = get_data_store().load()

    ensure_working(data)
    current_entry = data['work'][-1]

    ensure_end_after_start(current_entry, time)
    current_entry['end'] = time

    get_data_store().dump(data)

    print('You stopped working on ' + colorizer.red(current_entry['name']) + ' at ' +
          colorizer.yellow(formatted_str_for_isotime_str(time, '%H:%M')) + '.')



from tt.exceptz.exceptz import NoTask, BadArguments


def ensure_working(data):
    if data.get('work') and 'end' not in data['work'][-1]:
        return

    raise NoTask("For all I know, you aren't working on anything. "
                 "I don't know what to do.\n"
                 "See `tt -h` to know how to start working.")


def ensure_end_after_start(entry, end_time):
    if end_time <= entry['start']:
        raise BadArguments("The task is supposed to end before it has begun or at the same time "
                           "You seem confused. Now I'm confused. Aborting...")



class TIError(Exception):
    """Errors raised by TI."""


class AlreadyOn(TIError):
    """Already working on that task."""


class NoEditor(TIError):
    """No $EDITOR set."""


class InvalidYAML(TIError):
    """No $EDITOR set."""


class NoTask(TIError):
    """Not working on a task yet."""


class BadArguments(TIError):
    """The command line arguments passed are not valid."""


class BadTime(TIError):
    """Time string can't be parsed."""


class NonexistentDatasource(TIError):
    """The requested datasource is not supported"""



import json
from os import path

class JsonStore(object):

    def __init__(self, filename):
        self.filename = filename

    def load(self):

        if path.exists(self.filename):
            with open(self.filename) as f:
                data = json.load(f)

        else:
            data = {'work': []}

        return data

    def dump(self, data):
        with open(self.filename, 'w') as f:
            json.dump(data, f, separators=(',', ': '), indent=2)



from tt.exceptz.exceptz import AlreadyOn
from tt.dataaccess.utils import get_data_store
from tt.dateutils.dateutils import formatted_str_for_isotime_str

def action_start(colorizer, name, time):
    data = get_data_store().load()
    work = data['work']

    if work and 'end' not in work[-1]:
        raise AlreadyOn("You are already working on %s. Stop it or use a "
                        "different sheet." % (colorizer.yellow(work[-1]['name']),))

    entry = {
        'name': name,
        'start': time,
    }

    work.append(entry)
    get_data_store().dump(data)

    print('Started working on ' + colorizer.green(name) + ' at ' +
           colorizer.yellow(formatted_str_for_isotime_str(time, '%H:%M')) + '.')



from colorama import Fore
import re


color_regex = re.compile("(\x9B|\x1B\\[)[0-?]*[ -\/]*[@-~]")


def strip_color(str):
    """Strip color from string."""
    return color_regex.sub("", str)


def len_color(str):
    """Compute how long the color escape sequences in the string are."""
    return len(str) - len(strip_color(str))


def ljust_with_color(str, n):
    """ljust string that might contain color."""
    return str.ljust(n + len_color(str))


class Colorizer(object):
    def __init__(self, use_color):
        self.use_color = use_color

    def set_use_color(self, use_color):
        self.use_color = use_color

    def get_use_color(self):
        return self.use_color

    def red(self,str):
        if self.use_color:
            return Fore.RED + str + Fore.RESET
        else:
            return str

    def green(self,str):
        if self.use_color:
            return Fore.GREEN + str + Fore.RESET
        else:
            return str

    def yellow(self,str):
        if self.use_color:
            return Fore.YELLOW + str + Fore.RESET
        else:
            return str

    def blue(self,str):
        if self.use_color:
            return Fore.BLUE + str + Fore.RESET
        else:
            return str



from __future__ import print_function

from collections import defaultdict

from tt.dateutils.dateutils import *
from tt.dataaccess.utils import get_data_store
from tt.actions.utils import reportingutils


def action_report(colorizer, activity):
    print('Displaying all entries for ', colorizer.yellow(activity), ' grouped by day:', sep='')
    print()
    sep = ' | '
    data = get_data_store().load()
    work = data['work']
    report = defaultdict(lambda: {'sum': timedelta(), 'notes': '', 'weekday': '', 'start_time': None, 'end_time': None})

    total_time = 0
    for item in work:
        if item['name'] == activity and 'end' in item:
            start_time = parse_isotime(item['start'])
            end_time = parse_isotime(item['end'])
            day = reportingutils.extract_day(item['start'])
            duration = parse_isotime(item['end']) - parse_isotime(item['start'])
            report[day]['sum'] += duration
            report[day]['notes'] += reportingutils.get_notes_from_workitem(item)
            report[day]['weekday'] = reportingutils.extract_day_custom_formatter(item['start'], '%a')
            report[day]['start_time'] = get_min_date(report[day]['start_time'], start_time)
            report[day]['end_time'] = get_max_date(report[day]['end_time'], end_time)
            total_time += duration.seconds

    print('weekday', sep, 'date', sep, 'total duration', sep, 'start time', sep, 'end time', sep, 'break', sep,
          'description', sep)

    for date, details in sorted(report.items()):
        start_time = utc_to_local(details['start_time']).strftime("%H:%M")
        end_time = utc_to_local(details['end_time']).strftime("%H:%M")
        break_duration = get_break_duration(details['start_time'], details['end_time'], details['sum'])
        print(details['weekday'], sep, date, sep, start_time, sep, end_time, sep,
              format_time(break_duration,colorizer), sep, format_time(details['sum'],colorizer), sep, details['notes'], sep="")

    should_hours = 8 * len(report.items())
    should_hours_str = str(should_hours) + ':00'
    print()
    print('Based on your current entries, you should have logged ', colorizer.green(should_hours_str), ' ; you instead logged ',
          format_time_seconds(total_time,colorizer), sep='')


def get_break_duration(start_time, end_time, net_work_duration):
    total_work_duration = end_time - start_time
    return total_work_duration - net_work_duration


def format_time(duration_timedelta, colorizer):
    return format_time_seconds(duration_timedelta.seconds, colorizer)


def format_time_seconds(duration_secs, colorizer):
    hours, rem = divmod(duration_secs, 3600)
    mins, secs = divmod(rem, 60)
    formatted_time_str = str(hours).rjust(2, str('0')) + ':' + str(mins).rjust(2, str('0'))
    if hours >= 8:
        return colorizer.green(formatted_time_str)
    else:
        return colorizer.red(formatted_time_str)


def get_min_date(date_1, date_2):
    if date_1 is None:
        date_1 = parse_isotime('2099-01-01T00:00:00.000001Z')
    return date_1 if date_1 < date_2 else date_2


def get_max_date(date_1, date_2):
    if date_1 is None:
        date_1 = parse_isotime('2015-01-01T00:00:00.000001Z')
    return date_1 if date_1 > date_2 else date_2



from __future__ import print_function
from collections import defaultdict

from tt.dataaccess.utils import get_data_store
from tt.dateutils.dateutils import *
from tt.colors.colors import *


def action_log(period):
    data = get_data_store().load()
    work = data['work']
    log = defaultdict(lambda: {'delta': timedelta()})
    current = None

    for item in work:
        start_time = parse_isotime(item['start'])

        if 'end' in item:
            log[item['name']]['delta'] += (
                    parse_isotime(item['end']) - start_time)
        else:
            log[item['name']]['delta'] += datetime.utcnow() - start_time
            current = item['name']

    name_col_len = 0

    for name, item in log.items():
        name_col_len = max(name_col_len, len(strip_color(name)))

        secs = item['delta'].total_seconds()
        tmsg = []

        if secs > 3600:
            hours = int(secs // 3600)
            secs -= hours * 3600
            tmsg.append(str(hours) + ' hour' + ('s' if hours > 1 else ''))

        if secs > 60:
            mins = int(secs // 60)
            secs -= mins * 60
            tmsg.append(str(mins) + ' minute' + ('s' if mins > 1 else ''))

        if secs:
            tmsg.append(str(secs) + ' second' + ('s' if secs > 1 else ''))

        print(tmsg)
        log[name]['tmsg'] = ', '.join(tmsg)[::-1].replace(',', '& ', 1)[::-1]

    for name, item in sorted(log.items(), key=(lambda x: x[0]), reverse=True):
        print(ljust_with_color(name, name_col_len), ' :: ', item['tmsg'],
              end=' <- working\n' if current == name else '\n')



from __future__ import print_function

from tt.dataaccess.utils import get_data_store
from tt.dateutils.dateutils import *
from tt.actions.utils.utils import ensure_working


def action_status(colorizer):
    data = get_data_store().load()

    ensure_working(data)

    current = data['work'][-1]

    start_time = parse_isotime(current['start'])
    diff = timegap(start_time, datetime.utcnow())

    isotime_local = isotime_utc_to_local(current['start'])
    start_h_m = isotime_local.strftime('%H:%M')
    now_time_str = datetime.now().strftime('%H:%M');

    print('You have been working on {0} for {1}, since {2}; It is now {3}.'
          .format(colorizer.green(current['name']), colorizer.yellow(diff),
                  colorizer.yellow(start_h_m), colorizer.yellow(now_time_str)))

    if 'notes' in current:
        for note in current['notes']:
            print('  * ', note)



import pytz
import os
import re

from tzlocal import get_localzone
from datetime import datetime, timedelta
from tt.exceptz.exceptz import TIError


TT_TODAY_ENV_VAR = "TT_CURRENT_DAY"

def get_local_timezone():
    return get_localzone()

#   returns a datetime
def utc_to_local(utc_dt):
    local_tz = get_local_timezone()
    local_dt = utc_dt.replace(tzinfo=pytz.utc).astimezone(local_tz)
    return local_tz.normalize(local_dt)

#   returns a datetime
def isotime_utc_to_local(isotime_utc):
    return utc_to_local(parse_isotime(isotime_utc))


def parse_isotime(isotime_str):
    return datetime.strptime(isotime_str, '%Y-%m-%dT%H:%M:%S.%fZ')

#   returns a datetime for an input string
def to_datetime(timestr):
    return parse_time_h_m_to_iso(timestr).isoformat() + 'Z'


def local_to_utc(local_dt):
    local_dt_dst = get_local_timezone().localize(local_dt)
    utc_dt = local_dt_dst.astimezone(pytz.utc)
    return utc_dt.replace(tzinfo=None)


def formatted_str_for_isotime_str(isotime_str, format_str):
    localtime = isotime_utc_to_local(isotime_str)
    return localtime.strftime(format_str)


def get_current_day():
    today_value = os.getenv(TT_TODAY_ENV_VAR, None)
    return today_value


def get_now():
    return datetime.now()


def parse_time_multiformat(timestr):
    if timestr == "now":
        return get_now()
    for time_format in ["%H:%M", "%H%M"]:
        try:
            settime = datetime.strptime(timestr, time_format)
            return settime
        except Exception as keep_going:
            pass

    raise TIError("Can't parse your date string. Supported formats are 14:30 or 1430")


def get_current_year_local_tz():
    now = datetime.utcnow()
    now_local_dt = utc_to_local(now)
    return now_local_dt.strftime("%Y")


def parse_time_h_m_to_iso(timestr):
    now = get_now()
    
    try:
        settime = parse_time_multiformat(timestr)
        x = now.replace(hour=settime.hour, minute=settime.minute, second=0, microsecond=1)
        if get_current_day() is not None:
            currentday = datetime.strptime(get_current_day(), "%Y-%m-%d")
            y = x.replace(day=currentday.day, month=currentday.month, year=currentday.year)
            return local_to_utc(y)
        return local_to_utc(x)
    except Exception as e:
        print(e)

    raise TIError("Don't understand the time %r" % (timestr,))


def timegap(start_time, end_time):
    diff = end_time - start_time

    mins = diff.total_seconds() // 60

    if mins == 0:
        return 'less than a minute'
    elif mins == 1:
        return 'a minute'
    elif mins < 44:
        return '{} minutes'.format(mins)
    elif mins < 89:
        return 'about an hour'
    elif mins < 1439:
        return 'about {} hours'.format(mins // 60)
    elif mins < 2519:
        return 'about a day'
    elif mins < 43199:
        return 'about {} days'.format(mins // 1440)
    elif mins < 86399:
        return 'about a month'
    elif mins < 525599:
        return 'about {} months'.format(mins // 43200)
    else:
        return 'more than a year'



import yaml
import os
import tempfile
import subprocess

from tt.exceptz.exceptz import InvalidYAML
from tt.exceptz.exceptz import NoEditor
from tt.dataaccess.utils import get_data_store


def action_edit():
    if "EDITOR" not in os.environ:
        raise NoEditor("Please set the 'EDITOR' environment variable")

    store = get_data_store()

    data = store.load()
    yml = yaml.safe_dump(data, default_flow_style=False, allow_unicode=True)

    cmd = os.getenv('EDITOR')
    fd, temp_path = tempfile.mkstemp(suffix='.yml', prefix='tt.')
    with open(temp_path, "r+") as f:
        f.write(yml.replace('\n- ', '\n\n- '))
        f.seek(0)
        subprocess.check_call(cmd + ' ' + temp_path, shell=True)
        yml = f.read()
        f.truncate()
        f.close

    os.close(fd)
    os.remove(temp_path)

    try:
        data = yaml.load(yml, Loader=yaml.SafeLoader)
    except yaml.YAMLError as exc:
        raise InvalidYAML("Oops, that YAML doesn't appear to be valid!")

    store.dump(data)



#!/usr/bin/env python
import io
import sys


from setuptools import find_packages, setup


def read(*filenames, **kwargs):
    encoding = kwargs.get("encoding", "utf-8")
    sep = kwargs.get("sep", "\n")
    buf = []
    for filename in filenames:
        with io.open(filename, encoding=encoding) as f:
            buf.append(f.read())
    return sep.join(buf)


testing = bool({"pytest", "test"}.intersection(sys.argv))

setup(
    name="tt-time-tracker",
    version="1.0.7",
    author="Bogdan Maxim",
    author_email="bogdan.maxim@metamorphant.de",
    packages=find_packages(),
    include_package_data=True,
    scripts=[],
    url="https://github.com/dribnif/tt",
    description="A simple, precise command-line based time tracker",
    long_description=read("README.md", "CHANGES.md"),
    long_description_content_type='text/markdown',
    entry_points={
        "console_scripts": [
            "tt = tt.tt:main",
        ]
    },
    install_requires=["colorama", "pyyaml", 'tzlocal==2.1', 'pytz', 'mock', 'lark-parser'],
    setup_requires=["pytest-runner"] if testing else [],
    tests_require=["pytest", "cram", "pytest-cram"],
    extras_require={
        "docs": ["ghp-import", "pygreen"],
    },
    python_requires='>=3.6'
)



from tt.dataaccess.utils import get_data_store
from tt.actions.utils.utils import ensure_working


def action_note(colorizer, content):
    data = get_data_store().load()

    ensure_working(data)

    current = data['work'][-1]

    if 'notes' not in current:
        current['notes'] = [content]
    else:
        current['notes'].append(content)

    get_data_store().dump(data)

    print('Yep, noted to ' + colorizer.yellow(current['name']) + '.')



from unittest import TestCase
from tt.dateutils.dateutils import *
from tt.exceptz.exceptz import TIError
import mock

import pytz

test_timezone = "Europe/Berlin"


class TestDateutils(TestCase):

    @mock.patch('tt.dateutils.dateutils.get_local_timezone')
    def test_utc_to_local_cet_winter_time(self, mocked):
        mocked.return_value=pytz.timezone(test_timezone)
        test_time_as_datetime = parse_isotime("2018-02-01T18:00:00.000001Z")
        time_h_m = utc_to_local(test_time_as_datetime).strftime("%H:%M")
        self.assertEqual("19:00", time_h_m)

    @mock.patch('tt.dateutils.dateutils.get_local_timezone')
    def test_utc_to_local_cet_summer_time(self, mocked):
        mocked.return_value = pytz.timezone(test_timezone)
        test_time_as_datetime = parse_isotime("2018-06-01T18:00:00.000001Z")
        time_h_m = utc_to_local(test_time_as_datetime).strftime("%H:%M")
        self.assertEqual("20:00", time_h_m)

    @mock.patch('tt.dateutils.dateutils.get_local_timezone')
    def test_isotime_utc_to_local_cet_winter_time(self, mocked):
        mocked.return_value = pytz.timezone(test_timezone)
        isotime_local = isotime_utc_to_local("2018-02-01T17:00:00.000001Z")
        self.assertEqual("2018-02-01T18:00:00.000001+01:00", isotime_local.isoformat())

    @mock.patch('tt.dateutils.dateutils.get_local_timezone')
    def test_isotime_utc_to_local_cet_summer_time(self, mocked):
        mocked.return_value = pytz.timezone(test_timezone)
        isotime_local = isotime_utc_to_local("2018-06-01T17:00:00.000001Z")
        self.assertEqual("2018-06-01T19:00:00.000001+02:00", isotime_local.isoformat())

    def test_parse_isotime(self):
        now_really = datetime.now()
        isotime_now = now_really.isoformat()+"Z"
        isotime_parsed_from_string = parse_isotime(isotime_now)
        self.assertEqual(isotime_parsed_from_string, now_really)
        
    def test_parse_time_multiformat_colon_separated(self):
        time_parsed = parse_time_multiformat("20:15")
        self.assertEqual(20, time_parsed.hour)
        self.assertEqual(15, time_parsed.minute)

    def test_parse_time_multiformat_not_separated(self):
        time_parsed = parse_time_multiformat("2015")
        self.assertEqual(20, time_parsed.hour)
        self.assertEqual(15, time_parsed.minute)

    @mock.patch('tt.dateutils.dateutils.get_now')
    def test_parse_time_multiformat_with_now_string(self, mocked_datetime_now):
        mocked_datetime_now.return_value = datetime(2019, 3, 16, 20, 15)
        time_parsed = parse_time_multiformat("now")
        self.assertEqual(2019, time_parsed.year)
        self.assertEqual(3, time_parsed.month)
        self.assertEqual(16, time_parsed.day)
        self.assertEqual(20, time_parsed.hour)
        self.assertEqual(15, time_parsed.minute)

    def test_parse_time_multiformat_not_separated(self):
        self.assertRaises(TIError, parse_time_multiformat, "51n3p")

    @mock.patch('tt.dateutils.dateutils.get_local_timezone')
    @mock.patch('tt.dateutils.dateutils.get_current_day')
    def test_isotime_utc_to_local_cet_summer_time(self, mocked_current_day, mocked_timezone ):
        mocked_timezone.return_value = pytz.timezone(test_timezone)
        mocked_current_day.return_value = "2018-11-21"
        parsed_time=parse_time_h_m_to_iso("01:15")
        self.date_time_assertion_helper(parsed_time, 2018, 11, 21, 0, 15, 0, 1)
        
        
    @mock.patch('tt.dateutils.dateutils.get_local_timezone')
    @mock.patch('tt.dateutils.dateutils.get_current_day')
    def test_isotime_utc_to_local_cet_summer_time(self, mocked_current_day, mocked_timezone ):
        mocked_timezone.return_value = pytz.timezone(test_timezone)
        mocked_current_day.return_value = "2018-06-21"
        parsed_time=parse_time_h_m_to_iso("01:15")
        self.date_time_assertion_helper(parsed_time, 2018, 6, 20, 23, 15, 0, 1)
        
    @mock.patch('tt.dateutils.dateutils.get_local_timezone')
    @mock.patch('tt.dateutils.dateutils.get_current_day')
    def test_isotime_utc_to_local_cet_summer_time(self, mocked_current_day, mocked_timezone ):
        mocked_timezone.return_value = pytz.timezone(test_timezone)
        mocked_current_day.return_value = "2018-11-21"
        parsed_time=parse_time_h_m_to_iso("0115")
        self.date_time_assertion_helper(parsed_time, 2018, 11, 21, 0, 15, 0, 1)
        
        
    @mock.patch('tt.dateutils.dateutils.get_local_timezone')
    @mock.patch('tt.dateutils.dateutils.get_current_day')
    def test_isotime_utc_to_local_cet_summer_time(self, mocked_current_day, mocked_timezone ):
        mocked_timezone.return_value = pytz.timezone(test_timezone)
        mocked_current_day.return_value = "2018-06-21"
        parsed_time=parse_time_h_m_to_iso("0115")
        self.date_time_assertion_helper(parsed_time, 2018, 6, 20, 23, 15, 0, 1)
        
    def date_time_assertion_helper(self, parsed_time, year, month, day, hour, minute, second, microsecond):
        self.assertEqual(year,parsed_time.year)
        self.assertEqual(month,parsed_time.month)
        self.assertEqual(day,parsed_time.day)
        self.assertEqual(hour,parsed_time.hour)
        self.assertEqual(minute,parsed_time.minute)
        self.assertEqual(second,parsed_time.second)
        self.assertEqual(microsecond,parsed_time.microsecond)

